/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <stdlib.h>
#include "Log.h"
#include "Debug.h"
#include "Macro.h"

#ifdef __LOG_ENABLE

struct Struct_Categorie Liste_Categorie[] = {
	{CATEG_MAIN, "Main"},
	{CATEG_THREAD_DETECT, "Thread Detect"},
	{CATEG_THREAD_SCAN, "Thread Scan equipement"},
	{CATEG_THREAD_NOTIF_PORT, "Thread Notification Port"},
	{CATEG_THREAD_NOTIF_EQU, "Thread Notification équipement"},
	{CATEG_BOUCLE_COMMUNIC, "Thread Boucle Communication"},
	{CATEG_RAFFRAICHIR_EQU, "Thread Rafraichir Tout Equipement"},
	{CATEG_NESSUS, "CATEG_NESSUS"},
	{CATEG_THREAD_SNMP_LISTE_ALL, "CATEG_THREAD_SNMP_LISTE_ALL"},
	{CATEG_THREAD_SNMP_LISTE, "CATEG_THREAD_SNMP_LISTE"},
	{CATEG_BOUCLE_SCAN, "CATEG_BOUCLE_SCAN"},
	{CATEG_INFO_AGENT, "CATEG_INFO_AGENT"},
	{CATEG_PING_DETECT, "CATEG_PING_DETECT"},
	{CATEG_SCAN_PLAGE, "CATEG_SCAN_PLAGE"},
	{CATEG_THREAD_ETAT_EQUIPEMENT, "CATEG_THREAD_ETAT_EQUIPEMENT"},
	{CATEG_THREAD_WAKEUP, "CATEG_THREAD_WAKEUP"},
	{CATEG_IP_DYNAMIC, "CATEG_IP_DYNAMIC"}
};

GList    *Recupere_Glist_Categorie (struct struct_Cle_Privee *pCle)
{
	struct Struct_Liste_Equipement* Equipement = NULL;
	GList    *GList_Log = NULL;

	GList_Log = NULL;
	switch (pCle->Categorie) {
	    case CATEG_THREAD_SCAN:
		    Equipement = pCle->Pointeur_Data;
		    GList_Log = Equipement->GList_Log;
		    break;
	    case CATEG_BOUCLE_COMMUNIC:
		    GList_Log = GList_Log_Thread_Communication;
		    break;
	    case CATEG_THREAD_DETECT:
		    GList_Log = GList_Log_Thread_Detect;
		    break;
	    case CATEG_THREAD_NOTIF_EQU:
		    GList_Log = GList_Log_Thread_Notification_Equipement;
		    break;
	    case CATEG_THREAD_NOTIF_PORT:
		    GList_Log = GList_Log_Thread_Notification_Port;
		    break;
	    case CATEG_RAFFRAICHIR_EQU:
		    GList_Log = GList_Log_Thread_Raffraichi_Equipement;
		    break;
	    case CATEG_NESSUS:
#ifdef         __NESSUS_ENABLE
		    GList_Log = GList_Log_Thread_Nessus;
#endif
		    break;
	    case CATEG_THREAD_SNMP_LISTE_ALL:
#ifdef        __SNMP_ENABLE
		    GList_Log = GList_Log_Thread_Snmp_Liste_All;
#endif
		    break;
	    case CATEG_THREAD_SNMP_LISTE:
#ifdef        __SNMP_ENABLE
		    GList_Log = GList_Log_Thread_Snmp_Liste;
#endif
		    break;
	    case CATEG_BOUCLE_SCAN:
		    GList_Log = GList_Log_Boucle_Scan;
		    break;
	    case CATEG_INFO_AGENT:
		    GList_Log = GList_Log_Info_Agent;
		    break;
	    case CATEG_PING_DETECT:
		    GList_Log = GList_Log_Ping_Detect;
		    break;
	    case CATEG_SCAN_PLAGE:
		    GList_Log = GList_Log_Scanner_plage;
		    break;
	    case CATEG_THREAD_ETAT_EQUIPEMENT:
		    GList_Log = GList_Log_Thread_Etat_Equipement;
		    break;
	    case CATEG_THREAD_WAKEUP:
		    GList_Log = GList_Log_Thread_Envoie_WakeUp;
		    break;
	    case CATEG_IP_DYNAMIC:
		    GList_Log = GList_Log_Ip_Dynamic;
		    break;
		default :
			GList_Log = GList_Log_Main;
	}	

	return GList_Log;
}


void create_cle_privee (void)
{
	if (pthread_key_create (&Cle_privee_thread, NULL) == -1)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "pthread_key_create");
}


void Log_Equipement (struct Struct_Liste_Equipement *Equipement, gchar * Message)
{
	if ((Equipement == NULL) || (Message == NULL)) {
		printf ("Log_Equipement : Equipement==NULL)||(Message==NULL)\n");
		exit (EXIT_FAILURE);
	}
	PROTECT_CODE(Semaphore_Debug);
	G_LIST_APPEND (Equipement->GList_Log, g_strdup (Message));
	DEPROTECT_CODE(Semaphore_Debug);
}


void Initialisation_Log (void)
{
	init_backtrace();
	once_cle_privee = PTHREAD_ONCE_INIT;
	GList_Log_Main = NULL;
	G_LIST_APPEND (GList_Log_Main, "---------------------------------------");
	G_LIST_APPEND (GList_Log_Main, "----- LOG autoscan-network-daemon -----");
	G_LIST_APPEND (GList_Log_Main, "----------------- MAIN ----------------");
	G_LIST_APPEND (GList_Log_Main, "---------------------------------------");
	MALLOC(Donnee_Cle_Main,sizeof(struct struct_Cle_Privee));
	Donnee_Cle_Main->Categorie = CATEG_MAIN;
	Donnee_Cle_Main->Pointeur_Data = NULL;
	GList_Log_Thread_Communication = NULL;
	GList_Log_Thread_Detect = NULL;
	GList_Log_Thread_Notification_Equipement = NULL;
	GList_Log_Thread_Notification_Port = NULL;
	GList_Log_Thread_Raffraichi_Equipement = NULL;
#ifdef         __NESSUS_ENABLE
	GList_Log_Thread_Nessus = NULL;
#endif
#ifdef        __SNMP_ENABLE
	GList_Log_Thread_Snmp_Liste_All = NULL;
	GList_Log_Thread_Snmp_Liste = NULL;
#endif
	GList_Log_Boucle_Scan = NULL;
	GList_Log_Info_Agent = NULL;
	GList_Log_Ping_Detect = NULL;
	GList_Log_Scanner_plage = NULL;
	G_LIST_APPEND (GList_Log_Scanner_plage, "---------- Scanner Plage ----------");
	GList_Log_Thread_Etat_Equipement = NULL;
	G_LIST_APPEND (GList_Log_Thread_Etat_Equipement, "---------- Thread Etat Equipement ----------");
	GList_Log_Thread_Envoie_WakeUp = NULL;
	G_LIST_APPEND (GList_Log_Thread_Envoie_WakeUp, "---------- WakeUp ----------");
	GList_Log_Ip_Dynamic = NULL;
	G_LIST_APPEND (GList_Log_Ip_Dynamic, "---------- Ip Dynamic ----------");
}

#endif  


void Envoie_Log_Equipement (gpointer * Donnee)
{
	struct struct_Demande_Log_Equipement *Message = (struct struct_Demande_Log_Equipement *) Donnee;
	struct struct_Retour_Log_Equipement Message_Retour;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement, Nb_Log, Nb_Caract_Info, Log;
	gchar    *Info_Log = NULL;

	ERROR_if (Donnee == NULL);
#ifdef __LOG_ENABLE
	Recherche_Index_Equipement (Message->ID_Equipement, FALSE, &ID_Equipement);
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);


	bzero (&Message_Retour, sizeof (struct struct_Retour_Log_Equipement));
	Message_Retour.ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Message_Retour.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);


	Nb_Log = g_list_length (Equipement->GList_Log);

	Log = Nb_Log;
	strcpy (Message_Retour.Info, "");
	Nb_Caract_Info = 0;
	while ((Log > 1) && (Log + 40 != Nb_Log) && (Nb_Caract_Info < TAILLE_INFO_LOG_EQUIPEMENT)) {
		Info_Log = g_list_nth_data (Equipement->GList_Log, Log - 1);
		if ((strlen (Info_Log) + Nb_Caract_Info) > TAILLE_INFO_LOG_EQUIPEMENT)
			break;
		strcat (Message_Retour.Info, Info_Log);
		strcat (Message_Retour.Info, "\n");
		Nb_Caract_Info = Nb_Caract_Info + strlen (Info_Log) + 2;
		Log--;
	}

	Communication_Envoie_Donnee (RETOUR_LOG_EQUIPEMENT, (gpointer *) & Message_Retour, sizeof (Message_Retour));
#endif
}
