/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Notification_Equipement.h"
#include "Debug.h"
#include "Scan_Arp.h"
#include "Semaphore.h"
#include "Com/Initialisation_Com.h"
#include "Com/Commun.h"
#include "Thread.h"
#include "Macro.h"

void Envoie_Ping_Time_Equipement (struct Struct_Liste_Equipement *Equipement,gchar *Ping_Time)
{
	struct struct_Info_Ping_Time Message;

	ERROR_if (Equipement == NULL);
	ERROR_if (Ping_Time == NULL);
	bzero (&Message, sizeof (struct struct_Info_Ping_Time));
	Message.ID_Reseau = Info_Demarrage.ID_Reseau;
	memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
	if (strlen(Ping_Time)<TAILLE_MAX_CHAINE_INFO_PING_TIME)
		strcpy(Message.Info_Ping_Time,Ping_Time);
	else {
		strncpy(Message.Info_Ping_Time,Ping_Time,TAILLE_MAX_CHAINE_INFO_PING_TIME-1);
		Message.Info_Ping_Time[TAILLE_MAX_CHAINE_INFO_PING_TIME] = 0;
	}
	Communication_Envoie_Donnee (RETOUR_INFO_PING_TIME, (gpointer *) & Message, sizeof (Message));
}

void Thread_Notification_Equipement (void)
{
	struct Struct_Liste_Equipement *Equipement = NULL;
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;
#ifdef __LOG_ENABLE
	struct struct_Cle_Privee *Donnee_Cle = NULL;
#endif
	struct struct_Retour_Notification_Changement_Etat_Equipement Message;
	gint      Nb_Objet_Liste, i;

#ifdef __LOG_ENABLE
	GList_Log_Thread_Notification_Equipement = NULL; 
	G_LIST_APPEND (GList_Log_Thread_Notification_Equipement, "---------- Thread Notification Equipement ----------");

  	pthread_once(&once_cle_privee,create_cle_privee);
	MALLOC(Donnee_Cle,sizeof(struct struct_Cle_Privee));
	Donnee_Cle->Categorie = CATEG_THREAD_NOTIF_EQU;
	Donnee_Cle->Pointeur_Data = NULL;
	pthread_setspecific(Cle_privee_thread,Donnee_Cle);
#endif
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "");

	while (TRUE) {
		i = 0;
		Nb_Objet_Liste = g_list_length (GList_Notification_Equipement);
		while (i < Nb_Objet_Liste) {
		
			PROTECT_CODE (Semaphore_Notification_Equipement);
			Nb_Objet_Liste = g_list_length (GList_Notification_Equipement);
			if (i < Nb_Objet_Liste) {
				Notification_Equipement = g_list_nth_data (GList_Notification_Equipement, i);
				Equipement = Notification_Equipement->Equipement;
				if ((!Equipement->Rafraichissement_Etat_En_Cours) && (Equipement->ID_Equipement[6] != 0)) {
					if (Equipement->Compteur_Notification < TEMP_NOTIFICATION_EQUIPEMENT_CRITIQUE) {
						if (Equipement->Compteur_Notification >= TEMP_NOTIFICATION_EQUIPEMENT_CRITIQUE / 2)
							Envoie_Arp_Equipement (Equipement, 1);
						Equipement->Compteur_Notification++;
					}
					debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Thread notifie equipement : %s   Compteur : %d", Equipement->Adresse_Ip_Char, Equipement->Compteur_Notification);
					if (Equipement->Compteur_Notification == TEMP_NOTIFICATION_EQUIPEMENT_CRITIQUE) {
						Equipement->Compteur_Notification = 0;
						Equipement->Equipement_Allumer = FALSE;
						if (!Equipement->Alerte_Equipement_Active) {
							debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Notification critique");
							bzero (&Message, sizeof (struct struct_Retour_Notification_Changement_Etat_Equipement));
							Message.ID_Reseau = Info_Demarrage.ID_Reseau;
							memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
							Communication_Envoie_Donnee (RETOUR_NOTIFICATION_CHANGEMENT_ETAT_EQUIPEMENT, (gpointer *) & Message, sizeof (Message));
						}
						Equipement->Alerte_Equipement_Active = TRUE;
					}
				}
			}
			DEPROTECT_CODE (Semaphore_Notification_Equipement);
			i++;
		}
		sleep (1);
	}
}


void Thread_Rafraichir_Tout_Equipement (void)
{
	struct Struct_Liste_Equipement *Equipement = NULL;
	struct struct_Retour_Rachaichir_Etat_Equipement Message;
#ifdef __LOG_ENABLE
	struct struct_Cle_Privee *Donnee_Cle = NULL;
#endif
	gboolean  Sauve_Etat;
	gint      i,Timeout;;

#ifdef __LOG_ENABLE
	GList_Log_Thread_Raffraichi_Equipement = NULL; 
	G_LIST_APPEND (GList_Log_Thread_Raffraichi_Equipement, "---------- Thread Rafraichir Equipement ----------");

  	pthread_once(&once_cle_privee,create_cle_privee);
	MALLOC(Donnee_Cle,sizeof(struct struct_Cle_Privee));


	Donnee_Cle->Categorie = CATEG_RAFFRAICHIR_EQU;
	Donnee_Cle->Pointeur_Data = NULL;
	pthread_setspecific(Cle_privee_thread,Donnee_Cle);
#endif
	
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "");
	sleep (600);	
	while (1) {
		i = 0;
		Retour_Pointeur_Equipement (&Equipement,i);
		while (Equipement != NULL) {
			if ((!Equipement->Agent_AutoScan) && (!Equipement->Notification_Equipement) && (!Equipement->Rafraichissement_Etat_En_Cours)) {
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "On rafraichi %s\n",Equipement->Adresse_Ip_Char);
				Sauve_Etat = Equipement->Equipement_Allumer;
				Equipement->Compteur_Notification = 0;
				Equipement->Equipement_Allumer = FALSE;
				Equipement->Rafraichissement_Etat_En_Cours = TRUE;
			
				Envoie_Arp_Equipement (Equipement, 4);
			
				Ping_equipement(Equipement);
			
				Timeout = 0;
				while ((Timeout<5) && (Equipement->Equipement_Allumer)){
					sleep (1);
					Timeout++;
				}
				Equipement->Rafraichissement_Etat_En_Cours = FALSE;

				if (Sauve_Etat != Equipement->Equipement_Allumer) {
					bzero (&Message, sizeof (struct struct_Retour_Rachaichir_Etat_Equipement));
					Message.ID_Reseau = Info_Demarrage.ID_Reseau;
					memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
					Message.Equipement_Allumer = Equipement->Equipement_Allumer;
					Message.Derniere_Modification_Etat_Equipement = Equipement->Derniere_Modification_Etat_Equipement;
					Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, (gpointer *) & Message, sizeof (Message));
				}
			}
			i++;
			sleep (15);
			Retour_Pointeur_Equipement (&Equipement,i);
		}
		sleep (60);
	}
}

void Thread_Rafraichir_Etat_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct struct_Retour_Rachaichir_Etat_Equipement Message;
#ifdef __LOG_ENABLE
	struct struct_Cle_Privee *Donnee_Cle = NULL;	
#endif
	gboolean  Sauve_Etat;
	gint	Timeout;
	
#ifdef __LOG_ENABLE

  	pthread_once(&once_cle_privee,create_cle_privee);
	MALLOC(Donnee_Cle,sizeof(struct struct_Cle_Privee));
	Donnee_Cle->Categorie = CATEG_THREAD_ETAT_EQUIPEMENT;
	Donnee_Cle->Pointeur_Data = NULL;
	pthread_setspecific(Cle_privee_thread,Donnee_Cle);
#endif
	
	ERROR_if (Equipement == NULL);
	Sauve_Etat = Equipement->Equipement_Allumer;
	Equipement->Compteur_Notification = 0;
	Equipement->Equipement_Allumer = FALSE;
	Equipement->Rafraichissement_Etat_En_Cours = TRUE;

	Envoie_Arp_Equipement (Equipement, 4);

	Ping_equipement(Equipement);

	Timeout = 0;
	while ((Timeout<5) && (Equipement->Equipement_Allumer)){
		sleep (1);
		Timeout++;
	}
	Equipement->Rafraichissement_Etat_En_Cours = FALSE;

	if (Sauve_Etat != Equipement->Equipement_Allumer) {
		bzero (&Message, sizeof (struct struct_Retour_Rachaichir_Etat_Equipement));
		Message.ID_Reseau = Info_Demarrage.ID_Reseau;
		memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
		Message.Equipement_Allumer = Equipement->Equipement_Allumer;
		Message.Derniere_Modification_Etat_Equipement = Equipement->Derniere_Modification_Etat_Equipement;
		Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, (gpointer *) & Message, sizeof (Message));
	}
	debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin Thread_Rafraichir_Etat_Equipement");
	Stop_Thread (NULL);
}

void Rafraichir_Etat_Equipement (gpointer * Donnee)
{
	struct struct_Demande_Rachaichir_Etat_Equipement *Message = (struct struct_Demande_Rachaichir_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement, TRUE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Refresh state (Equipment unknown)");
		return;	
	}
	if (ID_Equipement == EQUIPEMENT_NO_IP) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Refresh state (no IP)");
		return;	
	}
	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);

	if (!Equipement->Notification_Equipement)
		demmarrage_Thread (&Thread_Rafraichir_Etat_Equipement, (gpointer *) Equipement); 
}


void Ajout_Notification_Equipement (gpointer * Donnee)
{
	struct struct_Demande_Notication_Critique_Etat_Equipement *Message = (struct struct_Demande_Notication_Critique_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement, FALSE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Add (Equipment unknown)");
		return;	
	}
	/*if (ID_Equipement == EQUIPEMENT_NO_IP) {
	 * debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Add (no IP)");
	 * return;
	 * } */

	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	Equipement->Alerte_Equipement_Active = FALSE;
	Ajout_Notification_Equipement_Glist (Equipement);
}

void Enlever_Notification_Equipement (gpointer * Donnee)
{
	struct struct_Annulation_Notification_Etat_Equipement *Message = (struct struct_Annulation_Notification_Etat_Equipement *) Donnee;
	struct Struct_Liste_Equipement *Equipement = NULL;
	gint      ID_Equipement;

	ERROR_if (Donnee == NULL);
	Recherche_Index_Equipement (Message->ID_Equipement, TRUE, &ID_Equipement);
	if (ID_Equipement == EQUIPEMENT_UNKNOWN) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Delete Notification (Equipment unknown)");
		return;	
	}
	if (ID_Equipement == EQUIPEMENT_NO_IP) {
		debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Delete Notification (no IP)");
		return;	
	}

	Retour_Pointeur_Equipement (&Equipement,ID_Equipement);
	Enlever_Notification_Equipement_Glist (Equipement);
}

void Affiche_Debug_Liste_Notification_Equipement_Securiser (void)
{
	PROTECT_CODE(Semaphore_Notification_Equipement);
	
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "************* Afficher liste notification equipement");
	Nb_Objet_Liste = g_list_length (GList_Notification_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Notification_Equipement = g_list_nth_data (GList_Notification_Equipement, i);
		debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Notification Equipement : %s [%s]", Notification_Equipement->Equipement->Adresse_Ip_Char, Notification_Equipement->Equipement->Adresse_MAC_Char);
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "************* fin");
	DEPROTECT_CODE(Semaphore_Notification_Equipement);
}

struct Struct_Liste_Equipement *Recupere_Pointeur_Notification_Equipement_Dans_Une_Liste (int rang)
{
	return g_list_nth_data (GList_Notification_Equipement, rang);
}

gint Recherche_Index_Notification_Equipement_Non_Securiser (struct Struct_Liste_Equipement * Equipement)
{			
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Recherche index");
	Nb_Objet_Liste = g_list_length (GList_Notification_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Notification_Equipement = g_list_nth_data (GList_Notification_Equipement, i);
		if (Notification_Equipement->Equipement == Equipement) {
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Notification trouve");
			return i;
		}
	}
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "index non trouve");
	return -1;	
}

void Enlever_Notification_Equipement_Glist (struct Struct_Liste_Equipement *Equipement)
{
	PROTECT_CODE(Semaphore_Notification_Equipement);
	gint      ID_Equipement;

	ERROR_if (Equipement == NULL);
	Equipement->Notification_Equipement = FALSE;
	if ((ID_Equipement = Recherche_Index_Notification_Equipement_Non_Securiser (Equipement)) != -1)
		GList_Notification_Equipement = g_list_remove (GList_Notification_Equipement, Recupere_Pointeur_Notification_Equipement_Dans_Une_Liste (ID_Equipement));
	else
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Enlever Notification Equipement n'existe pas: [%s]", Equipement->Adresse_MAC_Char);
	DEPROTECT_CODE(Semaphore_Notification_Equipement);
	Affiche_Debug_Liste_Notification_Equipement_Securiser ();
}


void Ajout_Notification_Equipement_Glist (struct Struct_Liste_Equipement *Equipement)
{
	PROTECT_CODE(Semaphore_Notification_Equipement);// Permet de securiser la suppression (ex si le thread de notification d'equipement est en train de lire la liste des notifications et quand meme temps on enleve un equipement dans cette liste il y aurait une erreur de segmentation)
	
	struct Struct_Liste_Notification_Equipement *Notification_Equipement = NULL;

	ERROR_if (Equipement == NULL);
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Ajout notification");
	Equipement->Notification_Equipement = TRUE;
	if (Recherche_Index_Notification_Equipement_Non_Securiser (Equipement) == -1) {
		MALLOC(Notification_Equipement,sizeof (struct Struct_Liste_Notification_Equipement));
	
	
	
		Notification_Equipement->Equipement = Equipement;
		Equipement->Compteur_Notification = 0;
		debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nouveau Notification Equipement : %s [%s]", Notification_Equipement->Equipement->Adresse_Ip_Char, Notification_Equipement->Equipement->Adresse_MAC_Char);
		G_LIST_APPEND (GList_Notification_Equipement, Notification_Equipement);
	}
	else
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nouveau Notification Equipement existe deja: [%s]", Equipement->Adresse_MAC_Char);
	DEPROTECT_CODE(Semaphore_Notification_Equipement);
	Affiche_Debug_Liste_Notification_Equipement_Securiser ();
}

void Initialisation_Notification_Equipement (void)
{
	GList_Notification_Equipement = NULL;
#ifdef __LOG_ENABLE
	GList_Log_Thread_Notification_Equipement = NULL;
    GList_Log_Thread_Raffraichi_Equipement = NULL;
#endif
}
